@@ -1,16 +1,18 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level
-#of this repository contains the full copyright notices and license terms.
+#This file is part of Tryton.  The COPYRIGHT file at the top level of
+#this repository contains the full copyright notices and license terms.
 "Purchase"
 from trytond.model import ModelWorkflow, ModelView, ModelSQL, fields
 from trytond.modules.company import CompanyReport
 from trytond.wizard import Wizard
 from trytond.backend import TableHandler
+from trytond.pyson import Not, Equal, Eval, Or, Bool, If, In, Get, And, \
+        PYSONEncoder
 from decimal import Decimal
 import datetime
 import copy
 
 _STATES = {
-    'readonly': "state != 'draft'",
+    'readonly': Not(Equal(Eval('state'), 'draft')),
 }
 
 
@@ -21,9 +23,12 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
 
     company = fields.Many2One('company.company', 'Company', required=True,
             states={
-                'readonly': "state != 'draft' or bool(lines)",
-            }, domain=["('id', 'company' in context and '=' or '!=', " \
-                    "context.get('company', 0))"])
+                'readonly': Or(Not(Equal(Eval('state'), 'draft')),
+                    Bool(Eval('lines'))),
+            }, domain=[
+                ('id', If(In('company', Eval('context', {})), '=', '!='),
+                    Get(Eval('context', {}), 'company', 0)),
+            ])
     reference = fields.Char('Reference', size=None, readonly=True, select=1)
     supplier_reference = fields.Char('Supplier Reference', select=1)
     description = fields.Char('Description', size=None, states=_STATES)
@@ -40,27 +45,28 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
     party = fields.Many2One('party.party', 'Party', change_default=True,
             required=True, states=_STATES, on_change=['party', 'payment_term'],
             select=1)
-    party_lang = fields.Function('get_function_fields', type='char',
-            string='Party Language', on_change_with=['party'])
+    party_lang = fields.Function(fields.Char('Party Language',
+        on_change_with=['party']), 'get_function_fields')
     invoice_address = fields.Many2One('party.address', 'Invoice Address',
-            domain=["('party', '=', party)"], states=_STATES)
+            domain=[('party', '=', Eval('party'))], states=_STATES)
     warehouse = fields.Many2One('stock.location', 'Warehouse',
             domain=[('type', '=', 'warehouse')], required=True, states=_STATES)
     currency = fields.Many2One('currency.currency', 'Currency', required=True,
         states={
-            'readonly': "state != 'draft' or (bool(lines) and bool(currency))",
+            'readonly': Or(Not(Equal(Eval('state'), 'draft')),
+                And(Bool(Eval('lines')), Bool(Eval('currency')))),
         })
-    currency_digits = fields.Function('get_function_fields', type='integer',
-            string='Currency Digits', on_change_with=['currency'])
+    currency_digits = fields.Function(fields.Integer('Currency Digits',
+        on_change_with=['currency']), 'get_function_fields')
     lines = fields.One2Many('purchase.line', 'purchase', 'Lines',
             states=_STATES, on_change=['lines', 'currency', 'party'])
     comment = fields.Text('Comment')
-    untaxed_amount = fields.Function('get_function_fields', type='numeric',
-            digits="(16, currency_digits)", string='Untaxed')
-    tax_amount = fields.Function('get_function_fields', type='numeric',
-            digits="(16, currency_digits)", string='Tax')
-    total_amount = fields.Function('get_function_fields', type='numeric',
-            digits="(16, currency_digits)", string='Total')
+    untaxed_amount = fields.Function(fields.Numeric('Untaxed',
+        digits=(16, Eval('currency_digits', 2))), 'get_function_fields')
+    tax_amount = fields.Function(fields.Numeric('Tax',
+        digits=(16, Eval('currency_digits', 2))), 'get_function_fields')
+    total_amount = fields.Function(fields.Numeric('Total',
+        digits=(16, Eval('currency_digits', 2))), 'get_function_fields')
     invoice_method = fields.Selection([
         ('manual', 'Manual'),
         ('order', 'Based On Order'),
@@ -80,24 +86,24 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
     invoices_recreated = fields.Many2Many(
             'purchase.purchase-recreated-account.invoice',
             'purchase', 'invoice', 'Recreated Invoices', readonly=True)
-    invoice_paid = fields.Function('get_function_fields', type='boolean',
-            string='Invoices Paid')
-    invoice_exception = fields.Function('get_function_fields', type='boolean',
-            string='Invoices Exception')
+    invoice_paid = fields.Function(fields.Boolean('Invoices Paid'),
+            'get_function_fields')
+    invoice_exception = fields.Function(fields.Boolean('Invoices Exception'),
+            'get_function_fields')
     shipment_state = fields.Selection([
         ('none', 'None'),
         ('waiting', 'Waiting'),
         ('received', 'Received'),
         ('exception', 'Exception'),
     ], 'Shipment State', readonly=True, required=True)
-    shipments = fields.Function('get_function_fields', type='many2many',
-            relation='stock.shipment.in', string='Shipments')
-    moves = fields.Function('get_function_fields', type='many2many',
-            relation='stock.move', string='Moves')
-    shipment_done = fields.Function('get_function_fields', type='boolean',
-            string='Shipment Done')
-    shipment_exception = fields.Function('get_function_fields', type='boolean',
-            string='Shipments Exception')
+    shipments = fields.Function(fields.One2Many('stock.shipment.in', None,
+        'Shipments'), 'get_function_fields')
+    moves = fields.Function(fields.One2Many('stock.move', None, 'Moves'),
+            'get_function_fields')
+    shipment_done = fields.Function(fields.Boolean('Shipment Done'),
+            'get_function_fields')
+    shipment_exception = fields.Function(fields.Boolean('Shipments Exception'),
+            'get_function_fields')
 
     def __init__(self):
         super(Purchase, self).__init__()
@@ -133,6 +139,10 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
                 "SET invoice_method = 'shipment' "\
                 "WHERE invoice_method = 'packing'")
 
+        # Add index on create_date
+        table = TableHandler(cursor, self, module_name)
+        table.index_action('create_date', action='add')
+
     def default_payment_term(self, cursor, user, context=None):
         payment_term_obj = self.pool.get('account.invoice.payment_term')
         payment_term_ids = payment_term_obj.search(cursor, user,
@@ -150,7 +160,6 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
         return False
 
     def default_company(self, cursor, user, context=None):
-        company_obj = self.pool.get('company.company')
         if context is None:
             context = {}
         if context.get('company'):
@@ -196,7 +205,7 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
     def default_shipment_state(self, cursor, user, context=None):
         return 'none'
 
-    def on_change_party(self, cursor, user, ids, vals, context=None):
+    def on_change_party(self, cursor, user, vals, context=None):
         party_obj = self.pool.get('party.party')
         address_obj = self.pool.get('party.address')
         payment_term_obj = self.pool.get('account.invoice.payment_term')
@@ -223,7 +232,7 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
                     res['payment_term'], context=context).rec_name
         return res
 
-    def on_change_with_currency_digits(self, cursor, user, ids, vals,
+    def on_change_with_currency_digits(self, cursor, user, vals,
             context=None):
         currency_obj = self.pool.get('currency.currency')
         if vals.get('currency'):
@@ -248,8 +257,7 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
             res[purchase.id] = purchase.currency.digits
         return res
 
-    def on_change_with_party_lang(self, cursor, user, ids, vals,
-            context=None):
+    def on_change_with_party_lang(self, cursor, user, vals, context=None):
         party_obj = self.pool.get('party.party')
         if vals.get('party'):
             party = party_obj.browse(cursor, user, vals['party'],
@@ -272,7 +280,7 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
                 res['language'] = purchase.party.lang.code
         return res
 
-    def on_change_lines(self, cursor, user, ids, vals, context=None):
+    def on_change_lines(self, cursor, user, vals, context=None):
         currency_obj = self.pool.get('currency.currency')
         tax_obj = self.pool.get('account.tax')
         invoice_obj = self.pool.get('account.invoice')
@@ -322,7 +330,7 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
                     res['total_amount'])
         return res
 
-    def get_function_fields(self, cursor, user, ids, names, args, context=None):
+    def get_function_fields(self, cursor, user, ids, names, context=None):
         '''
         Function to compute function fields for purchase ids.
 
@@ -609,7 +617,7 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
             res[purchase.id] = val
         return res
 
-    def get_rec_name(self, cursor, user, ids, name, arg, context=None):
+    def get_rec_name(self, cursor, user, ids, name, context=None):
         if not ids:
             return {}
         res = {}
@@ -618,20 +626,16 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
                     + ' - ' + purchase.party.name
         return res
 
-    def search_rec_name(self, cursor, user, name, args, context=None):
-        args2 = []
-        i = 0
-        while i < len(args):
-            names = args[i][2].split(' - ', 1)
-            ids = self.search(cursor, user, ['OR',
-                ('reference', args[i][1], names[0]),
-                ('supplier_reference', args[i][1], names[0]),
-                ], context=context)
-            args2.append(('id', 'in', ids))
-            if len(names) != 1 and names[1]:
-                args2.append(('party', args[i][1], names[1]))
-            i += 1
-        return args2
+    def search_rec_name(self, cursor, user, name, clause, context=None):
+        names = clause[2].split(' - ', 1)
+        ids = self.search(cursor, user, ['OR',
+            ('reference', clause[1], names[0]),
+            ('supplier_reference', clause[1], names[0]),
+            ], order=[], context=context)
+        res = [('id', 'in', ids)]
+        if len(names) != 1 and names[1]:
+            res.append(('party', clause[1], names[1]))
+        return res
 
     def copy(self, cursor, user, ids, default=None, context=None):
         if default is None:
@@ -654,14 +658,16 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
 
     def set_reference(self, cursor, user, purchase_id, context=None):
         sequence_obj = self.pool.get('ir.sequence')
+        config_obj = self.pool.get('purchase.configuration')
 
         purchase = self.browse(cursor, user, purchase_id, context=context)
 
         if purchase.reference:
             return True
 
-        reference = sequence_obj.get(cursor, user, 'purchase.purchase',
-                context=context)
+        config = config_obj.browse(cursor, user, 1, context=context)
+        reference = sequence_obj.get_id(cursor, user,
+                config.purchase_sequence.id, context=context)
         self.write(cursor, user, purchase_id, {
             'reference': reference,
             }, context=context)
@@ -846,56 +852,64 @@ class PurchaseLine(ModelSQL, ModelView):
         ('comment', 'Comment'),
         ], 'Type', select=1, required=True)
     quantity = fields.Float('Quantity',
-            digits="(16, unit_digits)",
+            digits=(16, Eval('unit_digits', 2)),
             states={
-                'invisible': "type != 'line'",
-                'required': "type == 'line'",
-                'readonly': "not globals().get('_parent_purchase')",
+                'invisible': Not(Equal(Eval('type'), 'line')),
+                'required': Equal(Eval('type'), 'line'),
+                'readonly': Not(Bool(Eval('_parent_purchase'))),
             }, on_change=['product', 'quantity', 'unit',
                 '_parent_purchase.currency', '_parent_purchase.party'])
     unit = fields.Many2One('product.uom', 'Unit',
             states={
-                'required': "product",
-                'invisible': "type != 'line'",
-                'readonly': "not globals().get('_parent_purchase')",
-            }, domain=["('category', '=', " \
-                    "(product, 'product.default_uom.category'))"],
-            context="{'category': (product, 'product.default_uom.category')}",
+                'required': Bool(Eval('product')),
+                'invisible': Not(Equal(Eval('type'), 'line')),
+                'readonly': Not(Bool(Eval('_parent_purchase'))),
+            }, domain=[
+                ('category', '=',
+                    (Eval('product'), 'product.default_uom.category')),
+            ],
+            context={
+                'category': (Eval('product'), 'product.default_uom.category'),
+            },
             on_change=['product', 'quantity', 'unit', '_parent_purchase.currency',
                 '_parent_purchase.party'])
-    unit_digits = fields.Function('get_unit_digits', type='integer',
-            string='Unit Digits', on_change_with=['unit'])
+    unit_digits = fields.Function(fields.Integer('Unit Digits',
+        on_change_with=['unit']), 'get_unit_digits')
     product = fields.Many2One('product.product', 'Product',
             domain=[('purchasable', '=', True)],
             states={
-                'invisible': "type != 'line'",
-                'readonly': "not globals().get('_parent_purchase')",
+                'invisible': Not(Equal(Eval('type'), 'line')),
+                'readonly': Not(Bool(Eval('_parent_purchase'))),
             }, on_change=['product', 'unit', 'quantity', 'description',
                 '_parent_purchase.party', '_parent_purchase.currency'],
-            context="{'locations': " \
-                        "_parent_purchase.warehouse and " \
-                        "[_parent_purchase.warehouse] or False, " \
-                    "'stock_date_end': _parent_purchase.purchase_date, " \
-                    "'purchasable': True, " \
-                    "'stock_skip_warehouse': True}")
+            context={
+                'locations': If(Bool(Get(Eval('_parent_purchase', {}),
+                    'warehouse')),
+                    [Get(Eval('_parent_purchase', {}), 'warehouse')],
+                    []),
+                'stock_date_end': Get(Eval('_parent_purchase', {}),
+                    'purchase_date'),
+                'purchasable': True,
+                'stock_skip_warehouse': True,
+            })
     unit_price = fields.Numeric('Unit Price', digits=(16, 4),
             states={
-                'invisible': "type != 'line'",
-                'required': "type == 'line'",
+                'invisible': Not(Equal(Eval('type'), 'line')),
+                'required': Equal(Eval('type'), 'line'),
             })
-    amount = fields.Function('get_amount', type='numeric', string='Amount',
-            digits="(16, _parent_purchase.currency_digits)",
-            states={
-                'invisible': "type not in ('line', 'subtotal')",
-                'readonly': "not globals().get('_parent_purchase')",
-            }, on_change_with=['type', 'quantity', 'unit_price',
-                '_parent_purchase.currency'])
+    amount = fields.Function(fields.Numeric('Amount',
+        digits=(16, Get(Eval('_parent_purchase', {}), 'currency_digits', 2)),
+        states={
+            'invisible': Not(In(Eval('type'), ['line', 'subtotal'])),
+            'readonly': Not(Bool(Eval('_parent_purchase'))),
+        }, on_change_with=['type', 'quantity', 'unit_price',
+            '_parent_purchase.currency']), 'get_amount')
     description = fields.Text('Description', size=None, required=True)
     note = fields.Text('Note')
     taxes = fields.Many2Many('purchase.line-account.tax',
             'line', 'tax', 'Taxes', domain=[('parent', '=', False)],
             states={
-                'invisible': "type != 'line'",
+                'invisible': Not(Equal(Eval('type'), 'line')),
             })
     invoice_lines = fields.Many2Many('purchase.line-account.invoice.line',
             'purchase_line', 'invoice_line', 'Invoice Lines', readonly=True)
@@ -905,10 +919,9 @@ class PurchaseLine(ModelSQL, ModelView):
             'purchase_line', 'move', 'Ignored Moves', readonly=True)
     moves_recreated = fields.Many2Many('purchase.line-recreated-stock.move',
             'purchase_line', 'move', 'Recreated Moves', readonly=True)
-    move_done = fields.Function('get_move_done', type='boolean',
-            string='Moves Done')
-    move_exception = fields.Function('get_move_exception', type='boolean',
-            string='Moves Exception')
+    move_done = fields.Function(fields.Boolean('Moves Done'), 'get_move_done')
+    move_exception = fields.Function(fields.Boolean('Moves Exception'),
+            'get_move_exception')
 
     def __init__(self):
         super(PurchaseLine, self).__init__()
@@ -940,7 +953,7 @@ class PurchaseLine(ModelSQL, ModelView):
     def default_unit_price(self, cursor, user, context=None):
         return Decimal('0.0')
 
-    def get_move_done(self, cursor, user, ids, name, args, context=None):
+    def get_move_done(self, cursor, user, ids, name, context=None):
         uom_obj = self.pool.get('product.uom')
         res = {}
         for line in self.browse(cursor, user, ids, context=context):
@@ -967,7 +980,7 @@ class PurchaseLine(ModelSQL, ModelView):
             res[line.id] = val
         return res
 
-    def get_move_exception(self, cursor, user, ids, name, args, context=None):
+    def get_move_exception(self, cursor, user, ids, name, context=None):
         res = {}
         for line in self.browse(cursor, user, ids, context=context):
             val = False
@@ -981,8 +994,7 @@ class PurchaseLine(ModelSQL, ModelView):
             res[line.id] = val
         return res
 
-    def on_change_with_unit_digits(self, cursor, user, ids, vals,
-            context=None):
+    def on_change_with_unit_digits(self, cursor, user, vals, context=None):
         uom_obj = self.pool.get('product.uom')
         if vals.get('unit'):
             uom = uom_obj.browse(cursor, user, vals['unit'],
@@ -990,7 +1002,7 @@ class PurchaseLine(ModelSQL, ModelView):
             return uom.digits
         return 2
 
-    def get_unit_digits(self, cursor, user, ids, name, arg, context=None):
+    def get_unit_digits(self, cursor, user, ids, name, context=None):
         res = {}
         for line in self.browse(cursor, user, ids, context=context):
             if line.unit:
@@ -1013,7 +1025,7 @@ class PurchaseLine(ModelSQL, ModelView):
         res = {}
         return res
 
-    def on_change_product(self, cursor, user, ids, vals, context=None):
+    def on_change_product(self, cursor, user, vals, context=None):
         party_obj = self.pool.get('party.party')
         product_obj = self.pool.get('product.product')
         uom_obj = self.pool.get('product.uom')
@@ -1080,11 +1092,11 @@ class PurchaseLine(ModelSQL, ModelView):
         vals = vals.copy()
         vals['unit_price'] = res['unit_price']
         vals['type'] = 'line'
-        res['amount'] = self.on_change_with_amount(cursor, user, ids,
-                vals, context=context)
+        res['amount'] = self.on_change_with_amount(cursor, user, vals,
+                context=context)
         return res
 
-    def on_change_quantity(self, cursor, user, ids, vals, context=None):
+    def on_change_quantity(self, cursor, user, vals, context=None):
         product_obj = self.pool.get('product.product')
 
         if context is None:
@@ -1108,10 +1120,10 @@ class PurchaseLine(ModelSQL, ModelView):
                 context=ctx2)[vals['product']]
         return res
 
-    def on_change_unit(self, cursor, user, ids, vals, context=None):
-        return self.on_change_quantity(cursor, user, ids, vals, context=context)
+    def on_change_unit(self, cursor, user, vals, context=None):
+        return self.on_change_quantity(cursor, user, vals, context=context)
 
-    def on_change_with_amount(self, cursor, user, ids, vals, context=None):
+    def on_change_with_amount(self, cursor, user, vals, context=None):
         currency_obj = self.pool.get('currency.currency')
         if vals.get('type') == 'line':
             if isinstance(vals.get('_parent_purchase.currency'), (int, long)):
@@ -1126,7 +1138,7 @@ class PurchaseLine(ModelSQL, ModelView):
             return amount
         return Decimal('0.0')
 
-    def get_amount(self, cursor, user, ids, name, arg, context=None):
+    def get_amount(self, cursor, user, ids, name, context=None):
         currency_obj = self.pool.get('currency.currency')
         res = {}
         for line in self.browse(cursor, user, ids, context=context):
@@ -1278,9 +1290,9 @@ class PurchaseLine(ModelSQL, ModelView):
         ctx['user'] = user
         move_id = move_obj.create(cursor, 0, vals, context=ctx)
 
-        self.write(cursor, user, line.id, {
+        self.write(cursor, 0, line.id, {
             'moves': [('add', move_id)],
-        }, context=context)
+        }, context=ctx)
         return move_id
 
 PurchaseLine()
@@ -1349,19 +1361,20 @@ class Template(ModelSQL, ModelView):
     _name = "product.template"
 
     purchasable = fields.Boolean('Purchasable', states={
-        'readonly': "active == False",
+        'readonly': Not(Bool(Eval('active'))),
         })
     product_suppliers = fields.One2Many('purchase.product_supplier',
             'product', 'Suppliers', states={
-                'readonly': "active == False",
-                'invisible': "(not purchasable) or (not company)",
+                'readonly': Not(Bool(Eval('active'))),
+                'invisible': Or(Not(Bool(Eval('purchasable'))),
+                    Not(Bool(Eval('company')))),
             })
     purchase_uom = fields.Many2One('product.uom', 'Purchase UOM', states={
-        'readonly': "active == False",
-        'invisible': "not purchasable",
-        'required': "purchasable",
-        }, domain=["('category', '=', (default_uom, 'uom.category'))"],
-        context="{'category': (default_uom, 'uom.category')}",
+        'readonly': Not(Bool(Eval('active'))),
+        'invisible': Not(Bool(Eval('purchasable'))),
+        'required': Bool(Eval('purchasable')),
+        }, domain=[('category', '=', (Eval('default_uom'), 'uom.category'))],
+        context={'category': (Eval('default_uom'), 'uom.category')},
         on_change_with=['default_uom', 'purchase_uom', 'purchasable'])
 
     def __init__(self):
@@ -1370,17 +1383,16 @@ class Template(ModelSQL, ModelView):
                 'change_purchase_uom': 'Purchase prices are based on the purchase uom, '\
                     'are you sure to change it?',
             })
-        if 'not bool(account_category) and bool(purchasable)' not in \
-                self.account_expense.states.get('required', ''):
-            self.account_expense = copy.copy(self.account_expense)
-            self.account_expense.states = copy.copy(self.account_expense.states)
-            if not self.account_expense.states.get('required'):
-                self.account_expense.states['required'] = \
-                        "not bool(account_category) and bool(purchasable)"
-            else:
-                self.account_expense.states['required'] = '(' + \
-                        self.account_expense.states['required'] + ') ' \
-                        'or (not bool(account_category) and bool(purchasable))'
+        self.account_expense = copy.copy(self.account_expense)
+        self.account_expense.states = copy.copy(self.account_expense.states)
+        required = And(Not(Bool(Eval('account_category'))),
+                Bool(Eval('purchasable')))
+        if not self.account_expense.states.get('required'):
+            self.account_expense.states['required'] = required
+        else:
+            self.account_expense.states['required'] = \
+                    Or(self.account_expense.states['required'],
+                            required)
         if 'account_category' not in self.account_expense.depends:
             self.account_expense = copy.copy(self.account_expense)
             self.account_expense.depends = \
@@ -1400,8 +1412,7 @@ class Template(ModelSQL, ModelView):
             return True
         return False
 
-    def on_change_with_purchase_uom(self, cursor, user, ids, vals,
-            context=None):
+    def on_change_with_purchase_uom(self, cursor, user, vals, context=None):
         uom_obj = self.pool.get('product.uom')
         res = False
 
@@ -1442,11 +1453,6 @@ Template()
 class Product(ModelSQL, ModelView):
     _name = 'product.product'
 
-    def on_change_with_purchase_uom(self, cursor, user, ids, vals, context=None):
-        template_obj = self.pool.get('product.template')
-        return template_obj.on_change_with_purchase_uom(cursor, user, ids, vals,
-                context=context)
-
     def get_purchase_price(self, cursor, user, ids, quantity=0, context=None):
         '''
         Return purchase price for product ids.
@@ -1526,8 +1532,10 @@ class ProductSupplier(ModelSQL, ModelView):
             'product_supplier', 'Prices')
     company = fields.Many2One('company.company', 'Company', required=True,
             ondelete='CASCADE', select=1,
-            domain=["('id', 'company' in context and '=' or '!=', " \
-                    "context.get('company', 0))"])
+            domain=[
+                ('id', If(In('company', Eval('context', {})), '=', '!='),
+                    Get(Eval('context', {}), 'company', 0)),
+            ])
     delivery_time = fields.Integer('Delivery Time',
             help="In number of days")
 
@@ -1536,7 +1544,6 @@ class ProductSupplier(ModelSQL, ModelView):
         self._order.insert(0, ('sequence', 'ASC'))
 
     def default_company(self, cursor, user, context=None):
-        company_obj = self.pool.get('company.company')
         if context is None:
             context = {}
         if context.get('company'):
@@ -1620,10 +1627,18 @@ class ShipmentIn(ModelSQL, ModelView):
     def __init__(self):
         super(ShipmentIn, self).__init__()
         self.incoming_moves = copy.copy(self.incoming_moves)
-        if "('supplier', '=', supplier)" not in self.incoming_moves.add_remove:
-            self.incoming_moves.add_remove = "[" + \
-                    self.incoming_moves.add_remove + ", " \
-                    "('supplier', '=', supplier)]"
+        add_remove = [
+            ('supplier', '=', Eval('supplier')),
+        ]
+        if not self.incoming_moves.add_remove:
+            self.incoming_moves.add_remove = add_remove
+        else:
+            self.incoming_moves.add_remove = \
+                    copy.copy(self.incoming_moves.add_remove)
+            self.incoming_moves.add_remove = [
+                add_remove,
+                self.incoming_moves.add_remove,
+            ]
         self._reset_columns()
 
         self._error_messages.update({
@@ -1674,55 +1689,44 @@ ShipmentIn()
 class Move(ModelSQL, ModelView):
     _name = 'stock.move'
 
-    purchase_line = fields.Many2One('purchase.line', select=1,
+    purchase_line = fields.Many2One('purchase.line', 'Purchase Line', select=1,
             states={
-                'readonly': "state != 'draft'",
+                'readonly': Not(Equal(Eval('state'), 'draft')),
             })
-    purchase = fields.Function('get_purchase', type='many2one',
-            relation='purchase.purchase', string='Purchase',
-            fnct_search='search_purchase', select=1, states={
-                'invisible': "not purchase_visible",
-            }, depends=['purchase_visible'])
-    purchase_quantity = fields.Function('get_purchase_fields',
-            type='float', digits="(16, unit_digits)",
-            string='Purchase Quantity',
-            states={
-                'invisible': "not purchase_visible",
-            }, depends=['purchase_visible'])
-    purchase_unit = fields.Function('get_purchase_fields',
-            type='many2one', relation='product.uom',
-            string='Purchase Unit',
-            states={
-                'invisible': "not purchase_visible",
-            }, depends=['purchase_visible'])
-    purchase_unit_digits = fields.Function('get_purchase_fields',
-            type='integer', string='Purchase Unit Digits')
-    purchase_unit_price = fields.Function('get_purchase_fields',
-            type='numeric', digits=(16, 4), string='Purchase Unit Price',
-            states={
-                'invisible': "not purchase_visible",
-            }, depends=['purchase_visible'])
-    purchase_currency = fields.Function('get_purchase_fields',
-            type='many2one', relation='currency.currency',
-            string='Purchase Currency',
-            states={
-                'invisible': "not purchase_visible",
-            }, depends=['purchase_visible'])
-    purchase_visible = fields.Function('get_purchase_visible',
-            type="boolean", string='Purchase Visible',
-            on_change_with=['from_location'])
-    supplier = fields.Function('get_supplier', type='many2one',
-            relation='party.party', string='Supplier',
-            fnct_search='search_supplier', select=1)
+    purchase = fields.Function(fields.Many2One('purchase.purchase', 'Purchase',
+        select=1, states={
+            'invisible': Not(Bool(Eval('purchase_visible'))),
+        }, depends=['purchase_visible']), 'get_purchase',
+        searcher='search_purchase')
+    purchase_quantity = fields.Function(fields.Float('Purchase Quantity',
+        digits=(16, Eval('unit_digits', 2)), states={
+            'invisible': Not(Bool(Eval('purchase_visible'))),
+        }, depends=['purchase_visible']), 'get_purchase_fields')
+    purchase_unit = fields.Function(fields.Many2One('product.uom',
+        'Purchase Unit', states={
+            'invisible': Not(Bool(Eval('purchase_visible'))),
+        }, depends=['purchase_visible']), 'get_purchase_fields')
+    purchase_unit_digits = fields.Function(fields.Integer(
+        'Purchase Unit Digits'), 'get_purchase_fields')
+    purchase_unit_price = fields.Function(fields.Numeric('Purchase Unit Price',
+        digits=(16, 4), states={
+            'invisible': Not(Bool(Eval('purchase_visible'))),
+        }, depends=['purchase_visible']), 'get_purchase_fields')
+    purchase_currency = fields.Function(fields.Many2One('currency.currency',
+        'Purchase Currency', states={
+            'invisible': Not(Bool(Eval('purchase_visible'))),
+        }, depends=['purchase_visible']), 'get_purchase_fields')
+    purchase_visible = fields.Function(fields.Boolean('Purchase Visible',
+        on_change_with=['from_location']), 'get_purchase_visible')
+    supplier = fields.Function(fields.Many2One('party.party', 'Supplier',
+        select=1), 'get_supplier', searcher='search_supplier')
+    purchase_exception_state = fields.Function(fields.Selection([
+        ('', ''),
+        ('ignored', 'Ignored'),
+        ('recreated', 'Recreated'),
+        ], 'Exception State'), 'get_purchase_exception_state')
 
-    purchase_exception_state = fields.Function('get_purchase_exception_state',
-            type='selection',
-            selection=[('', ''),
-                       ('ignored', 'Ignored'),
-                       ('recreated', 'Recreated')],
-            string='Exception State')
-
-    def get_purchase(self, cursor, user, ids, name, arg, context=None):
+    def get_purchase(self, cursor, user, ids, name, context=None):
         res = {}
         for move in self.browse(cursor, user, ids, context=context):
             res[move.id] = False
@@ -1730,8 +1734,8 @@ class Move(ModelSQL, ModelView):
                 res[move.id] = move.purchase_line.purchase.id
         return res
 
-    def get_purchase_exception_state(self, cursor, user, ids, name, arg,
-                                     context=None):
+    def get_purchase_exception_state(self, cursor, user, ids, name,
+            context=None):
         res = {}.fromkeys(ids, '')
         for move in self.browse(cursor, user, ids, context=context):
             if not move.purchase_line:
@@ -1742,16 +1746,10 @@ class Move(ModelSQL, ModelView):
                 res[move.id] = 'ignored'
         return res
 
-    def search_purchase(self, cursor, user, name, args, context=None):
-        args2 = []
-        i = 0
-        while i < len(args):
-            field = args[i][0]
-            args2.append(('purchase_line.' + field, args[i][1], args[i][2]))
-            i += 1
-        return args2
+    def search_purchase(self, cursor, user, name, clause, context=None):
+        return [('purchase_line.' + name,) + clause[1:]]
 
-    def get_purchase_fields(self, cursor, user, ids, names, arg, context=None):
+    def get_purchase_fields(self, cursor, user, ids, names, context=None):
         res = {}
         for name in names:
             res[name] = {}
@@ -1781,10 +1779,10 @@ class Move(ModelSQL, ModelView):
         vals = {
             'from_location': from_location,
         }
-        return self.on_change_with_purchase_visible(cursor, user, [], vals,
+        return self.on_change_with_purchase_visible(cursor, user, vals,
                 context=context)
 
-    def on_change_with_purchase_visible(self, cursor, user, ids, vals,
+    def on_change_with_purchase_visible(self, cursor, user, vals,
             context=None):
         location_obj = self.pool.get('stock.location')
         if vals.get('from_location'):
@@ -1794,7 +1792,7 @@ class Move(ModelSQL, ModelView):
                 return True
         return False
 
-    def get_purchase_visible(self, cursor, user, ids, name, arg, context=None):
+    def get_purchase_visible(self, cursor, user, ids, name, context=None):
         res = {}
         for move in self.browse(cursor, user, ids, context=context):
             res[move.id] = False
@@ -1802,7 +1800,7 @@ class Move(ModelSQL, ModelView):
                 res[move.id] = True
         return res
 
-    def get_supplier(self, cursor, user, ids, name, arg, context=None):
+    def get_supplier(self, cursor, user, ids, name, context=None):
         res = {}
         for move in self.browse(cursor, user, ids, context=context):
             res[move.id] = False
@@ -1810,14 +1808,8 @@ class Move(ModelSQL, ModelView):
                 res[move.id] = move.purchase_line.purchase.party.id
         return res
 
-    def search_supplier(self, cursor, user, name, args, context=None):
-        args2 = []
-        i = 0
-        while i < len(args):
-            args2.append(('purchase_line.purchase.party', args[i][1],
-                args[i][2]))
-            i += 1
-        return args2
+    def search_supplier(self, cursor, user, name, clause, context=None):
+        return [('purchase_line.purchase.party',) + clause[1:]]
 
     def write(self, cursor, user, ids, vals, context=None):
         purchase_obj = self.pool.get('purchase.purchase')
@@ -1869,12 +1861,11 @@ Move()
 class Invoice(ModelSQL, ModelView):
     _name = 'account.invoice'
 
-    purchase_exception_state = fields.Function('get_purchase_exception_state',
-            type='selection',
-            selection=[('', ''),
-                       ('ignored', 'Ignored'),
-                       ('recreated', 'Recreated')],
-            string='Exception State')
+    purchase_exception_state = fields.Function(fields.Selection([
+        ('', ''),
+        ('ignored', 'Ignored'),
+        ('recreated', 'Recreated'),
+        ], 'Exception State'), 'get_purchase_exception_state')
 
     def __init__(self):
         super(Invoice, self).__init__()
@@ -1896,8 +1887,8 @@ class Invoice(ModelSQL, ModelView):
         return super(Invoice, self).button_draft(
             cursor, user, ids, context=context)
 
-    def get_purchase_exception_state(self, cursor, user, ids, name, arg,
-                                     context=None):
+    def get_purchase_exception_state(self, cursor, user, ids, name,
+            context=None):
         purchase_obj = self.pool.get('purchase.purchase')
         purchase_ids = purchase_obj.search(
             cursor, user, [('invoices', 'in', ids)], context=context)
@@ -1951,19 +1942,13 @@ class OpenSupplier(Wizard):
         model_data_obj = self.pool.get('ir.model.data')
         act_window_obj = self.pool.get('ir.action.act_window')
         wizard_obj = self.pool.get('ir.action.wizard')
-
-        model_data_ids = model_data_obj.search(cursor, user, [
-            ('fs_id', '=', 'act_party_form'),
-            ('module', '=', 'party'),
-            ('inherit', '=', False),
-            ], limit=1, context=context)
-        model_data = model_data_obj.browse(cursor, user, model_data_ids[0],
-                context=context)
-        res = act_window_obj.read(cursor, user, model_data.db_id,
-                context=context)
+        act_window_id = model_data_obj.get_id(cursor, user, 'party',
+                'act_party_form', context=context)
+        res = act_window_obj.read(cursor, user, act_window_id, context=context)
         cursor.execute("SELECT DISTINCT(party) FROM purchase_purchase")
         supplier_ids = [line[0] for line in cursor.fetchall()]
-        res['domain'] = str([('id', 'in', supplier_ids)])
+        res['pyson_domain'] = PYSONEncoder().encode(
+                [('id', 'in', supplier_ids)])
 
         model_data_ids = model_data_obj.search(cursor, user, [
             ('fs_id', '=', 'act_open_supplier'),
@@ -1988,7 +1973,7 @@ class HandleShipmentExceptionAsk(ModelView):
 
     recreate_moves = fields.Many2Many(
         'stock.move', None, None, 'Recreate Moves',
-        domain=["('id', 'in', domain_moves)"], depends=['domain_moves'],
+        domain=[('id', 'in', Eval('domain_moves'))], depends=['domain_moves'],
         help='The selected moves will be recreated. '\
             'The other ones will be ignored.')
     domain_moves = fields.Many2Many(
@@ -2096,7 +2081,8 @@ class HandleInvoiceExceptionAsk(ModelView):
 
     recreate_invoices = fields.Many2Many(
         'account.invoice', None, None, 'Recreate Invoices',
-        domain=["('id', 'in', domain_invoices)"], depends=['domain_invoices'],
+        domain=[('id', 'in', Eval('domain_invoices'))],
+        depends=['domain_invoices'],
         help='The selected invoices will be recreated. '\
             'The other ones will be ignored.')
     domain_invoices = fields.Many2Many(
